@@ -1,128 +1,7 @@
 package JSON::Validator;
-
-=head1 NAME
-
-JSON::Validator - Validate data against a JSON schema
-
-=head1 VERSION
-
-0.65
-
-=head1 SYNOPSIS
-
-  use JSON::Validator;
-  my $validator = JSON::Validator->new;
-
-  # Define a schema - http://json-schema.org/examples.html
-  # You can also load schema from disk or web
-  $validator->schema(
-    {
-      type       => "object",
-      required   => ["firstName", "lastName"],
-      properties => {
-        firstName => {type => "string"},
-        lastName  => {type => "string"},
-        age       => {type => "integer", minimum => 0, description => "Age in years"}
-      }
-    }
-  );
-
-  # Validate your data
-  @errors = $validator->validate({firstName => "Jan Henning", lastName => "Thorsen", age => -42});
-
-  # Do something if any errors was found
-  die "@errors" if @errors;
-
-=head1 DESCRIPTION
-
-L<JSON::Validator> is a class for validating data against JSON schemas.
-You might want to use this instead of L<JSON::Schema> if you need to
-validate data against L<draft 4|https://github.com/json-schema/json-schema/tree/master/draft-04>
-of the specification.
-
-This module is currently EXPERIMENTAL. Hopefully nothing drastic will change,
-but it needs to fit together nicely with L<Swagger2> - Since this is a spin-off
-project.
-
-=head2 Supported schema formats
-
-L<JSON::Validator> can load JSON schemas in multiple formats: Plain perl data
-structured (as shown in L</SYNOPSIS>) or files on disk/web in the JSON/YAML
-format. The JSON parsing is done using L<Mojo::JSON>, while the YAML parsing
-is done with an optional modules which need to be installed manually.
-L<JSON::Validator> will look for the YAML modules in this order: L<YAML::XS>,
-L<YAML::Syck>. The order is set by which module that performs the best, so it
-might change in the future.
-
-=head2 Resources
-
-Here are some resources that are related to JSON schemas and validation:
-
-=over 4
-
-=item * L<http://json-schema.org/documentation.html>
-
-=item * L<http://spacetelescope.github.io/understanding-json-schema/index.html>
-
-=item * L<https://github.com/json-schema/json-schema/>
-
-=item * L<Swagger2>
-
-=back
-
-=head1 ERROR OBJECT
-
-=head2 Overview
-
-The method L</validate> and the function L</validate_json> returns
-error objects when the input data violates the L</schema>. Each of
-the objects looks like this:
-
-  bless {
-    message => "Some description",
-    path => "/json/path/to/node",
-  }, "JSON::Validator::Error"
-
-=head2 Operators
-
-The error object overloads the following operators:
-
-=over 4
-
-=item * bool
-
-Returns a true value.
-
-=item * string
-
-Returns the "path" and "message" part as a string: "$path: $message".
-
-=back
-
-=head2 Special cases
-
-Have a look at the L<test suite|https://github.com/jhthorsen/json-validator/tree/master/t>
-for documented examples of the error cases. Especially look at C<jv-allof.t>,
-C<jv-anyof.t> and C<jv-oneof.t>.
-
-The special cases for "allOf", "anyOf" and "oneOf" will contain the error messages
-from all the failing rules below. It can be a bit hard to read, so if the error message
-is long, then you might want to run a smaller test with C<JSON_VALIDATOR_DEBUG=1>.
-
-Example error object:
-
-  bless {
-    message => "[0] String is too long: 8/5. [1] Expected number - got string.",
-    path => "/json/path/to/node",
-  }, "JSON::Validator::Error"
-
-Note that these error messages are subject for change. Any suggestions are most
-welcome!
-
-=cut
-
 use Mojo::Base -base;
 use Exporter 'import';
+use JSON::Validator::Error;
 use Mojo::JSON;
 use Mojo::JSON::Pointer;
 use Mojo::URL;
@@ -140,100 +19,21 @@ use constant DEBUG => $ENV{JSON_VALIDATOR_DEBUG} || $ENV{SWAGGER2_DEBUG} || 0;
 use constant WARN_ON_MISSING_FORMAT => $ENV{JSON_VALIDATOR_WARN_ON_MISSING_FORMAT}
   || $ENV{SWAGGER2_WARN_ON_MISSING_FORMAT} ? 1 : 0;
 
-our $VERSION = '0.65';
-our @EXPORT_OK = qw( validate_json );
+our $VERSION = '0.67';
+our @EXPORT_OK = qw(validate_json);
 
 my $HTTP_SCHEME_RE = qr{^https?:};
 
-sub E { bless {path => $_[0] || '/', message => $_[1]}, 'JSON::Validator::Error'; }
+sub E { JSON::Validator::Error->new(@_) }
 sub S { Mojo::Util::md5_sum(Data::Dumper->new([@_])->Sortkeys(1)->Useqq(1)->Dump); }
 
-=head1 FUNCTIONS
-
-=head2 validate_json
-
-  use JSON::Validator "validate_json";
-  @errors = validate_json $data, $schema;
-
-This can be useful in web applications:
-
-  @errors = validate_json $c->req->json, "data://main/spec.json";
-
-See also L</validate> and L</ERROR OBJECT> for more details.
-
-=cut
-
 sub validate_json {
   __PACKAGE__->singleton->schema($_[1])->validate($_[0]);
 }
 
-=head1 ATTRIBUTES
-
-=head2 cache_dir
-
-  $self = $self->cache_dir($path);
-  $path = $self->cache_dir;
-
-Path to where downloaded spec files should be cached. Defaults to
-C<JSON_VALIDATOR_CACHE_DIR> or the bundled spec files that are shipped
-with this distribution.
-
-=head2 formats
-
-  $hash_ref = $self->formats;
-  $self = $self->formats(\%hash);
-
-Holds a hash-ref, where the keys are supported JSON type "formats", and
-the values holds a code block which can validate a given format.
-
-Note! The modules mentioned below are optional.
-
-=over 4
-
-=item * date-time
-
-An RFC3339 timestamp in UTC time. This is formatted as
-"YYYY-MM-DDThh:mm:ss.fffZ". The milliseconds portion (".fff") is optional
-
-=item * email
-
-Validated against the RFC5322 spec.
-
-=item * hostname
-
-Will be validated using L<Data::Validate::Domain> if installed.
-
-=item * ipv4
-
-Will be validated using L<Data::Validate::IP> if installed or
-fall back to a plain IPv4 IP regex.
-
-=item * ipv6
-
-Will be validated using L<Data::Validate::IP> if installed.
-
-=item * uri
-
-Validated against the RFC3986 spec.
-
-=back
-
-=head2 ua
-
-  $ua = $self->ua;
-  $self = $self->ua(Mojo::UserAgent->new);
-
-Holds a L<Mojo::UserAgent> object, used by L</schema> to load a JSON schema
-from remote location.
-
-Note that the default L<Mojo::UserAgent> will detect proxy settings and have
-L<Mojo::UserAgent/max_redirects> set to 3. (These settings are EXPERIMENTAL
-and might change without a warning)
-
-=cut
-
 has cache_dir => sub {
-  $ENV{JSON_VALIDATOR_CACHE_DIR} || File::Spec->catdir(File::Basename::dirname(__FILE__), qw( JSON Validator ));
+  $ENV{JSON_VALIDATOR_CACHE_DIR}
+    || File::Spec->catdir(File::Basename::dirname(__FILE__), qw(JSON Validator));
 };
 
 has formats => sub { shift->_build_formats };
@@ -246,75 +46,14 @@ has ua => sub {
   $ua;
 };
 
-=head1 METHODS
-
-=head2 coerce
-
-  $self = $self->coerce(booleans => 1, numbers => 1, strings => 1);
-  $self = $self->coerce({booleans => 1, numbers => 1, strings => 1});
-  $self = $self->coerce(1) # enable all
-  $hash = $self->coerce;
-
-Set the given type to coerce. Before enabling coercion this module is very
-strict when it comes to validating types. Example: The string C<"1"> is not
-the same as the number C<1>, unless you have coercion enabled.
-
-WARNING! Enabling coercion might hide bugs in your api, which would have been
-detected if you were strict. For example JavaScript is very picky on a number
-being an actual number. This module tries it best to convert the data on the
-fly into the proper value, but this means that you unit tests might be ok,
-but the client side libraries (that care about types) might break.
-
-Loading a YAML document will enable "booleans" automatically. This feature is
-experimental, but was added since YAML has no real concept of booleans, such
-as L<Mojo::JSON> or other JSON parsers.
-
-The coercion rules are EXPERIMENTAL and will be tighten/loosen if
-bugs are reported. See L<https://github.com/jhthorsen/json-validator/issues/8>
-for more details.
-
-=cut
-
 sub coerce {
   my $self = shift;
   return $self->{coerce} ||= {} unless @_;
-  $self->{coerce} = $_[0] eq '1' ? {booleans => 1, numbers => 1, strings => 1} : ref $_[0] ? {%{$_[0]}} : {@_};
+  $self->{coerce}
+    = $_[0] eq '1' ? {booleans => 1, numbers => 1, strings => 1} : ref $_[0] ? {%{$_[0]}} : {@_};
   $self;
 }
 
-=head2 schema
-
-  $self = $self->schema(\%schema);
-  $self = $self->schema($url);
-  $schema = $self->schema;
-
-Used to set a schema from either a data structure or a URL.
-
-C<$schema> will be a L<Mojo::JSON::Pointer> object when loaded,
-and C<undef> by default.
-
-The C<$url> can take many forms, but needs to point to a text file in the
-JSON or YAML format.
-
-=over 4
-
-=item * http://... or https://...
-
-A web resource will be fetched using the L<Mojo::UserAgent>, stored in L</ua>.
-
-=item * data://Some::Module/file.name
-
-This version will use L<Mojo::Loader/data_section> to load "file.name" from
-the module "Some::Module".
-
-=item * /path/to/file
-
-An URL (without a recognized scheme) will be loaded from disk.
-
-=back
-
-=cut
-
 sub schema {
   my ($self, $schema) = @_;
 
@@ -335,29 +74,11 @@ sub schema {
   $self;
 }
 
-=head2 singleton
-
-  $self = $class->singleton;
-
-Returns the L<JSON::Validator> object used by L</validate_json>.
-
-=cut
-
 sub singleton { state $validator = shift->new }
 
-=head2 validate
-
-  @errors = $self->validate($data);
-
-Validates C<$data> against a given JSON L</schema>. C<@errors> will
-contain validation error objects or be an empty list on success.
-
-See L</ERROR OBJECT> for details.
-
-=cut
-
 sub validate {
   my ($self, $data, $schema) = @_;
+  local $self->{seen} = {};
   $schema ||= $self->schema->data;    # back compat with Swagger2::SchemaValidator
   return E '/', 'No validation rules defined.' unless $schema and %$schema;
   return $self->_validate($data, '', $schema);
@@ -403,7 +124,8 @@ sub _load_schema {
 
   return $self->{cached}{$namespace} if $self->{cached}{$namespace};
   return eval {
-    warn "[JSON::Validator] Loading schema $url namespace=$namespace scheme=$scheme\n" if DEBUG;
+    warn "[JSON::Validator] Loading schema $url namespace=$namespace scheme=$scheme\n"
+      if DEBUG;
     $doc
       = $scheme eq 'file' ? Mojo::Util::slurp($namespace)
       : $scheme eq 'data' ? $self->_load_schema_from_data($url, $namespace)
@@ -491,7 +213,7 @@ sub _resolve_schema {
   # Seconds step: Resolve $ref
   for my $topic (@refs) {
     my $ref = $topic->{'$ref'} or next;    # already resolved?
-    $ref = "#/definitions/$ref" if $ref =~ /^\w+$/;                         # TODO: Figure out if this could be removed
+    $ref = "#/definitions/$ref" if $ref =~ /^\w+$/;    # TODO: Figure out if this could be removed
     $ref = Mojo::URL->new($namespace)->fragment($ref) if $ref =~ s!^\#!!;
     $ref = Mojo::URL->new($ref) unless ref $ref;
 
@@ -518,6 +240,9 @@ sub _validate {
   my $type = $schema->{type} || _guess_schema_type($schema, $data);
   my @errors;
 
+  # Avoid recursion
+  return if ref $data and $self->{seen}{"$schema\:$data"}++;
+
   # Make sure we validate plain data and not a perl object
   if (UNIVERSAL::can($data, 'TO_JSON')) {
     $data = $data->TO_JSON;
@@ -586,7 +311,8 @@ sub _merge_errors {
     $prefix = "/$prefix: " if $prefix;
 
     if (@e == grep { defined $_->[2] } @e) {
-      push @messages, sprintf '%sExpected %s - got %s.', $prefix, join(', ', map { $_->[2] } @e), $e[0][3];
+      push @messages, sprintf '%sExpected %s - got %s.', $prefix, join(', ', map { $_->[2] } @e),
+        $e[0][3];
     }
     else {
       push @messages, join ' ', map {"[$_->[0]] $prefix$_->[1]"} @e;
@@ -606,7 +332,8 @@ sub _validate_all_of {
   }
 
   warn "[JSON::Validator] allOf @{[$path||'/']} == [@errors]\n" if DEBUG == 2;
-  return E $path, sprintf 'allOf failed: %s', $self->_merge_errors($path, @errors) if grep {$_} @errors;
+  return E $path, sprintf 'allOf failed: %s', $self->_merge_errors($path, @errors)
+    if grep {$_} @errors;
   return;
 }
 
@@ -618,7 +345,8 @@ sub _validate_any_of {
   for my $rule (@$rules) {
     my @e = $self->_validate($data, $path, $rule);
     push @errors, @e ? @e : (undef);
-    $failed++ if @e;
+    last unless @e;
+    $failed++;
   }
 
   if ($failed < @$rules) {
@@ -627,7 +355,7 @@ sub _validate_any_of {
   }
   else {
     warn "[JSON::Validator] anyOf @{[$path||'/']} == [@errors]\n" if DEBUG == 2;
-    return E $path, sprintf 'anyOf failed: %s', $self->_merge_errors($path, @errors) if grep {$_} @errors;
+    return E $path, sprintf 'anyOf failed: %s', $self->_merge_errors($path, grep {$_} @errors);
   }
 }
 
@@ -647,9 +375,11 @@ sub _validate_one_of {
     return;
   }
 
-  warn "[JSON::Validator] oneOf @{[$path||'/']} == failed=$failed/@{[int @$rules]} / @errors\n" if DEBUG == 2;
+  warn "[JSON::Validator] oneOf @{[$path||'/']} == failed=$failed/@{[int @$rules]} / @errors\n"
+    if DEBUG == 2;
   return E $path, 'All of the oneOf rules match.' unless $failed;
-  return E $path, sprintf 'oneOf failed: %s', $self->_merge_errors($path, @errors) if grep {$_} @errors;
+  return E $path, sprintf 'oneOf failed: %s', $self->_merge_errors($path, @errors)
+    if grep {$_} @errors;
 }
 
 sub _validate_type_enum {
@@ -679,9 +409,7 @@ sub _validate_format {
   return E $path, "Does not match $schema->{format} format.";
 }
 
-sub _validate_type_any {
-  return;
-}
+sub _validate_type_any { }
 
 sub _validate_type_array {
   my ($self, $data, $path, $schema) = @_;
@@ -772,8 +500,12 @@ sub _validate_type_number {
   if (!defined $value or ref $value) {
     return E $path, _expected($expected => $value);
   }
-  unless (B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK) and 0 + $value eq $value and $value * 0 == 0) {
-    return E $path, "Expected $expected - got string." if !$self->{coerce}{numbers} or $value =~ /\D/;
+  unless (B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK)
+    and 0 + $value eq $value
+    and $value * 0 == 0)
+  {
+    return E $path, "Expected $expected - got string."
+      if !$self->{coerce}{numbers} or $value =~ /\D/;
     $_[1] = 0 + $value;    # coerce input value
   }
 
@@ -804,10 +536,12 @@ sub _validate_type_object {
     return E $path, _expected(object => $data);
   }
   if (defined $schema->{maxProperties} and $schema->{maxProperties} < keys %$data) {
-    push @errors, E $path, sprintf 'Too many properties: %s/%s.', int(keys %$data), $schema->{maxProperties};
+    push @errors, E $path, sprintf 'Too many properties: %s/%s.', int(keys %$data),
+      $schema->{maxProperties};
   }
   if (defined $schema->{minProperties} and $schema->{minProperties} > keys %$data) {
-    push @errors, E $path, sprintf 'Not enough properties: %s/%s.', int(keys %$data), $schema->{minProperties};
+    push @errors, E $path, sprintf 'Not enough properties: %s/%s.', int(keys %$data),
+      $schema->{minProperties};
   }
 
   while (my ($k, $r) = each %{$schema->{properties}}) {
@@ -858,7 +592,10 @@ sub _validate_type_string {
   if (!defined $value or ref $value) {
     return E $path, _expected(string => $value);
   }
-  if (B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK) and 0 + $value eq $value and $value * 0 == 0) {
+  if (  B::svref_2object(\$value)->FLAGS & (B::SVp_IOK | B::SVp_NOK)
+    and 0 + $value eq $value
+    and $value * 0 == 0)
+  {
     return E $path, "Expected string - got number." unless $self->{coerce}{strings};
     $_[1] = "$value";    # coerce input value
   }
@@ -867,12 +604,14 @@ sub _validate_type_string {
   }
   if (defined $schema->{maxLength}) {
     if (length($value) > $schema->{maxLength}) {
-      push @errors, E $path, sprintf "String is too long: %s/%s.", length($value), $schema->{maxLength};
+      push @errors, E $path, sprintf "String is too long: %s/%s.", length($value),
+        $schema->{maxLength};
     }
   }
   if (defined $schema->{minLength}) {
     if (length($value) < $schema->{minLength}) {
-      push @errors, E $path, sprintf "String is too short: %s/%s.", length($value), $schema->{minLength};
+      push @errors, E $path, sprintf "String is too short: %s/%s.", length($value),
+        $schema->{minLength};
     }
   }
   if (defined $schema->{pattern}) {
@@ -908,7 +647,10 @@ sub _guess_data_type {
   return lc $ref if $ref and !$blessed;
   return 'null' if !defined;
   return 'boolean' if $blessed and ("$_" eq "1" or !"$_");
-  return 'number' if B::svref_2object(\$_)->FLAGS & (B::SVp_IOK | B::SVp_NOK) and 0 + $_ eq $_ and $_ * 0 == 0;
+  return 'number'
+    if B::svref_2object(\$_)->FLAGS & (B::SVp_IOK | B::SVp_NOK)
+    and 0 + $_ eq $_
+    and $_ * 0 == 0;
   return $blessed || 'string';
 }
 
@@ -916,13 +658,17 @@ sub _guess_schema_type {
   return _guessed_right($_[1], 'object') if $_[0]->{additionalProperties};
   return _guessed_right($_[1], 'object') if $_[0]->{patternProperties};
   return _guessed_right($_[1], 'object') if $_[0]->{properties};
-  return _guessed_right($_[1], 'object') if defined $_[0]->{maxProperties} or defined $_[0]->{minProperties};
+  return _guessed_right($_[1], 'object')
+    if defined $_[0]->{maxProperties}
+    or defined $_[0]->{minProperties};
   return _guessed_right($_[1], 'array')  if $_[0]->{additionalItems};
   return _guessed_right($_[1], 'array')  if $_[0]->{items};
   return _guessed_right($_[1], 'array')  if $_[0]->{uniqueItems};
   return _guessed_right($_[1], 'array')  if defined $_[0]->{maxItems} or defined $_[0]->{minItems};
   return _guessed_right($_[1], 'string') if $_[0]->{pattern};
-  return _guessed_right($_[1], 'string') if defined $_[0]->{maxLength} or defined $_[0]->{minLength};
+  return _guessed_right($_[1], 'string')
+    if defined $_[0]->{maxLength}
+    or defined $_[0]->{minLength};
   return _guessed_right($_[1], 'number') if $_[0]->{multipleOf};
   return _guessed_right($_[1], 'number') if defined $_[0]->{maximum} or defined $_[0]->{minimum};
   return 'enum' if $_[0]->{enum};
@@ -934,7 +680,9 @@ sub _guessed_right {
   return _guess_data_type($_[0]) eq $_[1] ? $_[1] : undef;
 }
 
-sub _is_date_time { $_[0] =~ qr/^(\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+(?:\.\d+)?)(?:Z|([+-])(\d+):(\d+))?$/io; }
+sub _is_date_time {
+  $_[0] =~ qr/^(\d+)-(\d+)-(\d+)T(\d+):(\d+):(\d+(?:\.\d+)?)(?:Z|([+-])(\d+):(\d+))?$/io;
+}
 sub _is_domain { warn "Data::Validate::Domain is not installed"; return; }
 
 sub _is_email {
@@ -972,9 +720,10 @@ sub _is_uri { $_[0] =~ qr!^(([^:/?#]+):)?(//([^/?#]*))?([^?#]*)(\?([^#]*))?(#(.*
 # https://github.com/jhthorsen/json-validator/issues
 sub _load_yaml {
   require List::Util;
-  my @YAML_MODULES = qw( YAML::XS YAML::Syck );                                     # subject to change
+  my @YAML_MODULES = qw(YAML::XS YAML::Syck);    # subject to change
   my $YAML_MODULE = (List::Util::first { eval "require $_;1" } @YAML_MODULES)[0];
-  die "Need to install one of these YAML modules: @YAML_MODULES (YAML::XS is recommended)" unless $YAML_MODULE;
+  die "Need to install one of these YAML modules: @YAML_MODULES (YAML::XS is recommended)"
+    unless $YAML_MODULE;
   warn "[JSON::Validator] Using $YAML_MODULE to parse YAML\n" if DEBUG;
   Mojo::Util::monkey_patch(__PACKAGE__, _load_yaml => eval "\\\&$YAML_MODULE\::Load");
   _load_yaml(@_);
@@ -987,13 +736,279 @@ sub _path {
   "$_[0]/$_";
 }
 
-package    # hide from
-  JSON::Validator::Error;
+1;
 
-use overload q("") => sub { sprintf '%s: %s', @{$_[0]}{qw( path message )} }, bool => sub {1}, fallback => 1;
-sub message { shift->{message} }
-sub path    { shift->{path} }
-sub TO_JSON { {message => $_[0]->{message}, path => $_[0]->{path}} }
+=encoding utf8
+
+=head1 NAME
+
+JSON::Validator - Validate data against a JSON schema
+
+=head1 VERSION
+
+0.67
+
+=head1 SYNOPSIS
+
+  use JSON::Validator;
+  my $validator = JSON::Validator->new;
+
+  # Define a schema - http://json-schema.org/examples.html
+  # You can also load schema from disk or web
+  $validator->schema(
+    {
+      type       => "object",
+      required   => ["firstName", "lastName"],
+      properties => {
+        firstName => {type => "string"},
+        lastName  => {type => "string"},
+        age       => {type => "integer", minimum => 0, description => "Age in years"}
+      }
+    }
+  );
+
+  # Validate your data
+  @errors = $validator->validate({firstName => "Jan Henning", lastName => "Thorsen", age => -42});
+
+  # Do something if any errors was found
+  die "@errors" if @errors;
+
+=head1 DESCRIPTION
+
+L<JSON::Validator> is a class for validating data against JSON schemas.
+You might want to use this instead of L<JSON::Schema> if you need to
+validate data against L<draft 4|https://github.com/json-schema/json-schema/tree/master/draft-04>
+of the specification.
+
+This module is currently EXPERIMENTAL. Hopefully nothing drastic will change,
+but it needs to fit together nicely with L<Swagger2> - Since this is a spin-off
+project.
+
+=head2 Supported schema formats
+
+L<JSON::Validator> can load JSON schemas in multiple formats: Plain perl data
+structured (as shown in L</SYNOPSIS>) or files on disk/web in the JSON/YAML
+format. The JSON parsing is done using L<Mojo::JSON>, while the YAML parsing
+is done with an optional modules which need to be installed manually.
+L<JSON::Validator> will look for the YAML modules in this order: L<YAML::XS>,
+L<YAML::Syck>. The order is set by which module that performs the best, so it
+might change in the future.
+
+=head2 Resources
+
+Here are some resources that are related to JSON schemas and validation:
+
+=over 4
+
+=item * L<http://json-schema.org/documentation.html>
+
+=item * L<http://spacetelescope.github.io/understanding-json-schema/index.html>
+
+=item * L<https://github.com/json-schema/json-schema/>
+
+=item * L<Swagger2>
+
+=back
+
+=head1 ERROR OBJECT
+
+=head2 Overview
+
+The method L</validate> and the function L</validate_json> returns
+error objects when the input data violates the L</schema>. Each of
+the objects looks like this:
+
+  bless {
+    message => "Some description",
+    path => "/json/path/to/node",
+  }, "JSON::Validator::Error"
+
+See also L<JSON::Validator::Error>.
+
+=head2 Operators
+
+The error object overloads the following operators:
+
+=over 4
+
+=item * bool
+
+Returns a true value.
+
+=item * string
+
+Returns the "path" and "message" part as a string: "$path: $message".
+
+=back
+
+=head2 Special cases
+
+Have a look at the L<test suite|https://github.com/jhthorsen/json-validator/tree/master/t>
+for documented examples of the error cases. Especially look at C<jv-allof.t>,
+C<jv-anyof.t> and C<jv-oneof.t>.
+
+The special cases for "allOf", "anyOf" and "oneOf" will contain the error messages
+from all the failing rules below. It can be a bit hard to read, so if the error message
+is long, then you might want to run a smaller test with C<JSON_VALIDATOR_DEBUG=1>.
+
+Example error object:
+
+  bless {
+    message => "[0] String is too long: 8/5. [1] Expected number - got string.",
+    path => "/json/path/to/node",
+  }, "JSON::Validator::Error"
+
+Note that these error messages are subject for change. Any suggestions are most
+welcome!
+
+=head1 FUNCTIONS
+
+=head2 validate_json
+
+  use JSON::Validator "validate_json";
+  @errors = validate_json $data, $schema;
+
+This can be useful in web applications:
+
+  @errors = validate_json $c->req->json, "data://main/spec.json";
+
+See also L</validate> and L</ERROR OBJECT> for more details.
+
+=head1 ATTRIBUTES
+
+=head2 cache_dir
+
+  $self = $self->cache_dir($path);
+  $path = $self->cache_dir;
+
+Path to where downloaded spec files should be cached. Defaults to
+C<JSON_VALIDATOR_CACHE_DIR> or the bundled spec files that are shipped
+with this distribution.
+
+=head2 formats
+
+  $hash_ref = $self->formats;
+  $self = $self->formats(\%hash);
+
+Holds a hash-ref, where the keys are supported JSON type "formats", and
+the values holds a code block which can validate a given format.
+
+Note! The modules mentioned below are optional.
+
+=over 4
+
+=item * date-time
+
+An RFC3339 timestamp in UTC time. This is formatted as
+"YYYY-MM-DDThh:mm:ss.fffZ". The milliseconds portion (".fff") is optional
+
+=item * email
+
+Validated against the RFC5322 spec.
+
+=item * hostname
+
+Will be validated using L<Data::Validate::Domain> if installed.
+
+=item * ipv4
+
+Will be validated using L<Data::Validate::IP> if installed or
+fall back to a plain IPv4 IP regex.
+
+=item * ipv6
+
+Will be validated using L<Data::Validate::IP> if installed.
+
+=item * uri
+
+Validated against the RFC3986 spec.
+
+=back
+
+=head2 ua
+
+  $ua = $self->ua;
+  $self = $self->ua(Mojo::UserAgent->new);
+
+Holds a L<Mojo::UserAgent> object, used by L</schema> to load a JSON schema
+from remote location.
+
+Note that the default L<Mojo::UserAgent> will detect proxy settings and have
+L<Mojo::UserAgent/max_redirects> set to 3. (These settings are EXPERIMENTAL
+and might change without a warning)
+
+=head1 METHODS
+
+=head2 coerce
+
+  $self = $self->coerce(booleans => 1, numbers => 1, strings => 1);
+  $self = $self->coerce({booleans => 1, numbers => 1, strings => 1});
+  $self = $self->coerce(1) # enable all
+  $hash = $self->coerce;
+
+Set the given type to coerce. Before enabling coercion this module is very
+strict when it comes to validating types. Example: The string C<"1"> is not
+the same as the number C<1>, unless you have coercion enabled.
+
+WARNING! Enabling coercion might hide bugs in your api, which would have been
+detected if you were strict. For example JavaScript is very picky on a number
+being an actual number. This module tries it best to convert the data on the
+fly into the proper value, but this means that you unit tests might be ok,
+but the client side libraries (that care about types) might break.
+
+Loading a YAML document will enable "booleans" automatically. This feature is
+experimental, but was added since YAML has no real concept of booleans, such
+as L<Mojo::JSON> or other JSON parsers.
+
+The coercion rules are EXPERIMENTAL and will be tighten/loosen if
+bugs are reported. See L<https://github.com/jhthorsen/json-validator/issues/8>
+for more details.
+
+=head2 schema
+
+  $self = $self->schema(\%schema);
+  $self = $self->schema($url);
+  $schema = $self->schema;
+
+Used to set a schema from either a data structure or a URL.
+
+C<$schema> will be a L<Mojo::JSON::Pointer> object when loaded,
+and C<undef> by default.
+
+The C<$url> can take many forms, but needs to point to a text file in the
+JSON or YAML format.
+
+=over 4
+
+=item * http://... or https://...
+
+A web resource will be fetched using the L<Mojo::UserAgent>, stored in L</ua>.
+
+=item * data://Some::Module/file.name
+
+This version will use L<Mojo::Loader/data_section> to load "file.name" from
+the module "Some::Module".
+
+=item * /path/to/file
+
+An URL (without a recognized scheme) will be loaded from disk.
+
+=back
+
+=head2 singleton
+
+  $self = $class->singleton;
+
+Returns the L<JSON::Validator> object used by L</validate_json>.
+
+=head2 validate
+
+  @errors = $self->validate($data);
+
+Validates C<$data> against a given JSON L</schema>. C<@errors> will
+contain validation error objects or be an empty list on success.
+
+See L</ERROR OBJECT> for details.
 
 =head1 COPYRIGHT AND LICENSE
 
@@ -1007,5 +1022,3 @@ the terms of the Artistic License version 2.0.
 Jan Henning Thorsen - C<jhthorsen@cpan.org>
 
 =cut
-
-1;
